@@ -1,4 +1,19 @@
-/* compile with -DNO_GUI to have it work postscript_only without GUI at all
+/* 
+compile with -DTTY to have it output text-only tree without GUI
+
+compile with -DADDROOT to have it root the unrooted input tree and write tree on stdout (no GUI)
+
+compile with -DNO_GUI to have it work postscript_only without GUI at all
+
+WIN_MOTIF resources:
+Vibrant.systemfont : helvetica,14,b
+Njplot.systemfont : times,14,b
+Vibrant.programfont : fixed,14
+placed in file
+$HOME/.Xdefaults
+$XAPPLRESDIR/Vibrant
+/usr/X11R6/lib/X11/app-defaults/Vibrant    (Linux)
+/usr/openwin/lib/X11/app-defaults/Vibrant   (Solaris)
 */
 #include <math.h>
 #include <time.h>
@@ -6,6 +21,9 @@
 
 #define MAX_FRAC 0.95
 
+#if defined(TTY) || defined (ADDROOT)
+#define NO_GUI
+#endif
 #ifdef NO_GUI
 
 #define FALSE 0
@@ -24,10 +42,12 @@
 #endif
 
 #ifdef WIN_MAC
-#include <Quickdraw.h>
-#include <Files.h>
-#include <Scrap.h>
-#include <Resources.h>
+
+#ifdef verify
+#undef verify
+#endif
+#include <Carbon.h>
+//#include <MacHeadersCarbon.h>  // precompiled version
 #define myWaitAndProcessNextEvent Nlm_ProcessAnEvent
 extern int Nlm_textScrapFull;
 
@@ -44,6 +64,8 @@ char *get_prog_dir(void);
 #include <X11/Intrinsic.h>
 
 extern XtAppContext Nlm_appContext;
+extern Nlm_CharPtr Nlm_XrmGetResource(const Nlm_Char PNTR _resource);
+void set_systemfont(char *systemfont);
 
 void myWaitAndProcessNextEvent (void)
 {
@@ -88,6 +110,7 @@ typedef struct {
 	Nlm_MenU paper_menu;
 	Nlm_IteM save_plot_button;
 	Nlm_IteM save_tree_button;
+	Nlm_IteM SearchItem;
 	Nlm_IteM AgainItem;
 	Nlm_ChoicE choix_font, choix_taille;
 	Nlm_IteM bold_item, italic_item;
@@ -98,6 +121,8 @@ typedef struct {
 	Nlm_ButtoN subtree_up_button;
 	Nlm_IteM exit_button;
 	Nlm_PaneL tree_plot;
+	float zoomvalue;
+	Nlm_PrompT zoomprompt;
 	char *tree_name;
 } FD_nj_plot;
 
@@ -150,7 +175,7 @@ typedef struct _moments {
 /* prototypes of included functions */
 #ifndef NO_GUI
 FD_nj_plot *create_win_nj_plot(void);
-void cre_help_win(ButtoN unused);
+void cre_help_win(IteM item);
 FILE *open_path(char *fname);
 void topic_callback(LisT list);
 void help_ok_callback(ButtoN bouton);
@@ -185,7 +210,10 @@ void process_keys(Nlm_Char key);
 void change_page_count(ButtoN item);
 void page_count_callback(IteM item);
 void clear_tree(IteM);
+void scrollnotu(int count);
 void paste_tree(IteM);
+void scrollcallback(Nlm_BaR b, Nlm_SlatE g, Nlm_Int2 before, Nlm_Int2 after);
+void zoomcallback(Nlm_SwitcH s, Nlm_Int2 before, Nlm_Int2 after);
 #ifdef MICRO
 void copy_plot(IteM);
 void print_title(int x, int y, char *text, Nlm_FonT title_font, int p,int totp);
@@ -193,6 +221,7 @@ void print_plot(IteM);
 #endif
 #ifdef WIN_MAC
 extern void add_apropos(char *);
+int is_macosx(void);
 int crefpict(char *, PicHandle );
 void example_tree(IteM unused);
 void pict_plot(void);
@@ -205,10 +234,11 @@ extern int MG_GetOutputFName(char *fname, int maxl, char *dfault);
 
 #endif /* NO_GUI */
 
-void process_args(int *argc, char *argv[]);
+void process_args(int *argc, char *argv[], char **pred);
 void prepare_fonts(void);
 void prepare_ps_font(int fontnum, int use_bold, int use_italic);
 int direct_ps_plot(void);
+int calc_otu(struct noeud *pere, struct noeud *centre);
 void do_plot(void);
 int calc_text_size(char *text, int *pheight, int *pascent);
 void plotstring(char *nom);
@@ -226,6 +256,8 @@ void loadphylip(char *arbre);
 struct noeud *unrootedset(char *deb, char *fin, branche **p_int_br);
 char *extract_filename(char *name);
 char *nextpar(char *pospar);
+void make_binary_or_unrooted(char *arbre);
+int make_binary(char *arbre, char *debut, char *fin, int go_down);
 void mydrawstring(double x, double y, char *nom, char option);
 void moveto(double x,double y);
 void lineto(double x,double y);
@@ -259,6 +291,7 @@ int calc_n_desc(struct noeud *pere);
 void bad_format(char *);
 void majuscules(char *p);
 void postscript_plot(void);
+void tty_plot(void);
 
 
 
@@ -318,6 +351,10 @@ int doing_copy = FALSE;
 int doing_print = FALSE;
 float postscript_ratio = 0.60;
 
+char **tty_page;
+int tty_x, tty_y;
+char *outotu = NULL; /* value of optional argument -outotu name */
+
 
 #ifdef WIN_MAC
 Rect		myrect;
@@ -329,6 +366,7 @@ int 		margin;
 int ps_plot_only = TRUE;
 int main(int argc, char **argv)
 {
+char *red;
 #else
 int ps_plot_only = FALSE;
 
@@ -342,15 +380,15 @@ char **argv;
 
 Nlm_Int2 Nlm_Main(void)
 {
+char *red;
 #ifndef hpux
 argc = Nlm_GetArgc();
 argv = Nlm_GetArgv();
 #endif
-
 #endif /* NO_GUI */
 
 *fname = 0;
-process_args(&argc, argv);
+process_args(&argc, argv, &red);
 if(argc>=2) {
 	strcpy(fname,argv[1]);
 	}
@@ -367,6 +405,14 @@ putenv("LC_NUMERIC=C");
 	}
 #ifndef NO_GUI
    Nlm_WatchCursor();
+#ifdef WIN_MOTIF
+	{char *font;
+	font = Nlm_XrmGetResource("systemfont");
+	if(font != NULL) set_systemfont(font);
+	font = Nlm_XrmGetResource("programfont");
+	if(font != NULL) Nlm_programFont = Nlm_ParseFont(font); 
+	}
+#endif
    fd_nj_plot = create_win_nj_plot();
    prepare_fonts();
    fd_nj_plot->tree_name = fname;
@@ -378,6 +424,7 @@ putenv("LC_NUMERIC=C");
    	p_rect = (RecT *)GetWindowExtra(fd_nj_plot->nj_plot);
    	ObjectRect(fd_nj_plot->nj_plot, p_rect);
    }
+   if(red != NULL) search_callback( (Nlm_IteM)red );
    Nlm_ArrowCursor();
    ProcessEvents();
 #endif
@@ -398,6 +445,7 @@ FD_nj_plot *create_win_nj_plot(void)
 Nlm_Handle obj; int left, right;
   
 #ifdef WIN_MAC
+long gval;
 add_apropos("njplot");
 #endif
 Nlm_KeyboardView(process_keys);
@@ -430,6 +478,9 @@ fdui->nj_plot = win = DocumentWindow(-50, -33, -3, -3, "Njplot",
   fdui->save_tree_button = CommandItem(menu, "Save Tree", save_tree_callback);
 #ifdef MICRO
   CommandItem(menu, PRINT_LABEL, print_plot);
+#endif
+#ifdef WIN_MAC
+if( ! is_macosx() )
 #endif
   CommandItem(menu, QUIT_LABEL, exit_callback);
   Advance(PLACE_OF_MENUS);
@@ -444,9 +495,9 @@ fdui->nj_plot = win = DocumentWindow(-50, -33, -3, -3, "Njplot",
   CommandItem(menu, "Example", example_tree);
 #endif
   SeparatorItem(menu);
-  CommandItem(menu, FIND_LABEL, search_callback);
+  fdui->SearchItem = CommandItem(menu, FIND_LABEL, search_callback);
   fdui->AgainItem = CommandItem(menu, AGAIN_LABEL, search_callback);
-  Disable(fdui->AgainItem);
+/*  Disable(fdui->AgainItem); */
   Advance(PLACE_OF_MENUS);
   fdui->tree_font_menu = PulldownMenu(PLACE_OF_MENUS, "Font"); 
   fdui->choix_font = ChoiceGroup(fdui->tree_font_menu, font_callback);
@@ -493,7 +544,12 @@ fdui->nj_plot = win = DocumentWindow(-50, -33, -3, -3, "Njplot",
 	sprintf(nom, "Page count (%d)", page_count);
   	CommandItem(fdui->paper_menu, nom, page_count_callback);
   }
-#endif  	
+#endif  
+
+  Advance(PLACE_OF_MENUS);
+  obj = PulldownMenu(PLACE_OF_MENUS, "Help");
+  CommandItem(obj, "Help", cre_help_win);
+
   Break(PLACE_OF_MENUS);
   fdui->choix_group = NormalGroup(win, 4, 1, "Operation", programFont, operation_callback);
   RadioButton(fdui->choix_group, "Full tree");
@@ -511,27 +567,28 @@ fdui->nj_plot = win = DocumentWindow(-50, -33, -3, -3, "Njplot",
 			toggle_bootstrap_callback);
   SetStatus(fdui->bootstrap_button, show_bootstrap);
 
+  Advance(win);
+  group = NormalGroup(win, 2, 1, "Zoom", programFont, NULL);
+obj = Nlm_UpDownSwitch(group, 0, zoomcallback);
+fdui->zoomvalue = 1.;
+Nlm_SetSwitchParams( (Nlm_SwitcH)obj, 1, 100);
+fdui->zoomprompt = Nlm_StaticPrompt(group, "100%", 35, Nlm_stdLineHeight, Nlm_programFont, 'l');
+
   Advance(win);
   GetNextPosition(win, &position);
   position.x += 5;
   position.y += 10;
   SetNextPosition(win, position);
-  fdui->subtree_up_button = PushButton(win, "Subtree Up", subtree_up_callback);
+  obj = fdui->subtree_up_button = PushButton(win, "Subtree Up", subtree_up_callback);
   Disable(fdui->subtree_up_button);
+  
 
-  Advance(win);
-#ifdef WIN_MAC
-  GetNextPosition(win, &position);
-  position.x += 5;
-  SetNextPosition(win, position);
-#endif
-  obj = Nlm_PushButton(win, "Help", cre_help_win);
+Break(win);
 Nlm_GetPosition(obj, &winrect); 
 right = winrect.right;
-Nlm_GetPosition(group, &winrect); 
-left = winrect.left;
+Nlm_GetNextPosition(win, &position); 
+left = position.x;
 
-	Break(win);
 #ifdef WIN_MSWIN
 #define PLOT_HEIGHT 430
 #elif defined(WIN_MAC)
@@ -539,9 +596,30 @@ left = winrect.left;
 #else
 #define PLOT_HEIGHT 350
 #endif
-fdui->tree_plot = AutonomousPanel(win, right - left + 1, PLOT_HEIGHT, 
-	tree_draw_proc, NULL, NULL, 0, NULL, NULL);
+
+fdui->tree_plot = Nlm_AutonomousPanel(win, right - left + 1, PLOT_HEIGHT, 
+		tree_draw_proc, scrollcallback, NULL, sizeof(int), NULL, NULL); 
+	{int hidebar = TRUE; /* put TRUE iff scrollbar should be hidden */
+	Nlm_SetPanelExtra(fdui->tree_plot, &hidebar);
+	}
 SetPanelClick(fdui->tree_plot, tree_click_proc, NULL, NULL, NULL);
+obj = Nlm_GetSlateVScrollBar((SlatE)fdui->tree_plot);
+Nlm_Hide(obj);
+Nlm_SetBarValue(obj, 0);
+Nlm_SetBarMax(obj, 1);
+
+{int space = 0; /* some space at the bottom of the window */
+#if defined(__APPLE__) || defined(WIN_MAC)
+space = Nlm_hScrollBarHeight;
+#elif defined(WIN_MOTIF)
+space = 3;
+#endif
+if(space > 0) {
+	Nlm_Break(win);
+	Nlm_StaticPrompt(win, "", 4, space, Nlm_programFont, 'l');
+	}
+}
+
 winrect.left = -1;
 SetWindowExtra(win, &winrect, NULL);
 #ifdef WIN_MAC
@@ -660,7 +738,7 @@ FILE *open_path(char *fname)  /* to open in read-only file fname searching for
 }
 
 
-void cre_help_win(ButtoN bouton)
+void cre_help_win(IteM bouton)
 {
 static WindoW win;
 LisT topic_list;
@@ -683,7 +761,7 @@ if(first) {
 		return;
 		}
 	first = FALSE;
-	win = DocumentWindow( -50, -33, -3, -3, "Njplot Help", NULL, NULL);
+	win = DocumentWindow( -50, -33, -3, -3, "Njplot Help", (Nlm_WndActnProc)help_ok_callback, NULL);
 	group = HiddenGroup(win, -1, -2, NULL);
 	SetGroupSpacing(group, 0, 5);
 	topic_list = SingleList(group, 7, 8, topic_callback);
@@ -814,6 +892,7 @@ if(subtree_center != NULL) {
 		return;
 		}
 	subtree_notu = calc_n_desc(subtree_center) - 1;
+	scrollnotu(subtree_notu);
 	}
 need_runtree = TRUE;
 tree_draw_proc(fd_nj_plot->tree_plot);
@@ -836,7 +915,7 @@ else if( test_key('o') )
 else if( test_key('s') )
 	save_plot_callback(fd_nj_plot->save_plot_button);
 else if( test_key('f') )
-	search_callback(NULL);
+	search_callback(fd_nj_plot->SearchItem);
 else if( test_key('a') ) {
 	if(Enabled(fd_nj_plot->AgainItem) ) 
 		search_callback(fd_nj_plot->AgainItem);
@@ -885,20 +964,29 @@ void change_panel_size(PaneL panel, int change_x, int change_y)
 RecT r, r_group;
 GrouP group;
 Nlm_Boolean in_group;
+Nlm_BaR bar;
+int visible;
+static int recursive = FALSE;
 
+GetPosition(panel, &r);
+#ifdef WIN_MOTIF
+if(recursive) return;
+recursive = TRUE;
+bar = Nlm_GetSlateVScrollBar( (Nlm_SlatE)panel);
+visible = Nlm_Visible(bar);
 group = Parent(panel);
 in_group = (group != ParentWindow(panel));
-GetPosition(panel, &r);
 if(in_group) {
 	GetPosition(group, &r_group);
 	Hide(group);
 	}
 else
-	Hide(panel);
-Reset(panel);
+	Hide(panel); 
+#endif
 r.bottom += change_y;
 r.right += change_x;
 SetPosition(panel, &r);
+#ifdef WIN_MOTIF
 if(in_group) {
 	r_group.bottom += change_y;
 	r_group.right += change_x;
@@ -906,6 +994,9 @@ if(in_group) {
 	Show(group);
 	}
 else	Show(panel);
+if( !visible) Nlm_Hide(bar); 
+recursive = FALSE;
+#endif
 }
 
 
@@ -1062,8 +1153,9 @@ else if(choix == depl_racine)
 	new_outgroup_callback();
 else if(choix == subtree)
 	subtree_callback();
-else
+else {
 	show_tree_callback();
+	}
 }
 
 void subtree_callback(void)
@@ -1137,6 +1229,8 @@ void show_tree_callback(void)
 		}
 	Enable(fd_nj_plot->reroot_button);
 need_runtree = TRUE;
+subtree_notu = notu;
+if(subtree_center != NULL) scrollnotu(notu);
 subtree_center = NULL;
 tree_draw_proc(fd_nj_plot->tree_plot);
 }
@@ -1153,13 +1247,16 @@ void tree_draw_proc(PaneL panel)
 RecT ob_rect;
 Int2 width, height;
 static Int2 previous_h = 0, previous_w = 0;
+Nlm_BaR bar;
+int hidebar;
 
 WatchCursor();
+bar = Nlm_GetSlateVScrollBar((Nlm_SlatE)panel);
+Nlm_GetPanelExtra(panel, &hidebar);
+if(hidebar && Nlm_Visible(bar)) Nlm_Hide(bar);
+else if( !hidebar && !Nlm_Visible(bar)) Nlm_Show(bar);
 Select (panel);
 ObjectRect(panel, &ob_rect);
-#ifdef WIN_MOTIF
-ob_rect.bottom -= 3;
-#endif
 EraseRect(&ob_rect);
 FrameRect(&ob_rect);
 Nlm_ClipRect(&ob_rect);
@@ -1227,6 +1324,7 @@ void tree_click_proc(PaneL panel, PoinT click)
 		subtree_center = tabtax[node_num - 1];
 		subtree_ascend = subtree_center->v3;
 		subtree_notu = calc_n_desc(subtree_center) - 1;
+		scrollnotu(subtree_notu);
 		choix = show_tree;
 		SetValue(fd_nj_plot->choix_group, 0);
 		Nlm_InvalObject(fd_nj_plot->choix_group);
@@ -1243,6 +1341,7 @@ void tree_click_proc(PaneL panel, PoinT click)
 	tree_draw_proc(panel);
 }
 
+
 void paste_tree(IteM unused)
 {
 FILE *tmp;
@@ -1343,12 +1442,12 @@ void search_callback(IteM item)
 static WindoW win;
 static Nlm_Boolean search_done;
 static TexT select_box;
-static char select[100];
-char aux[100];
+static char select[500];
+char aux[500];
 int num, trouve;
 static int first = TRUE;
 
-if(item != fd_nj_plot->AgainItem) {
+if(item == fd_nj_plot->SearchItem) {
 	if(first) {
 		win = FixedWindow(-50,-50,-5,-5,"Name searched:",NULL);
 		select_box=DialogText(win,"",15,NULL);
@@ -1366,9 +1465,13 @@ if(item != fd_nj_plot->AgainItem) {
 	GetTitle(select_box, select, sizeof(select));
 	majuscules(select);
 	Hide(win);
-	Enable(fd_nj_plot->AgainItem);
+/*	Enable(fd_nj_plot->AgainItem); */
 	Select(fd_nj_plot->nj_plot);
 	}
+else if(item != fd_nj_plot->AgainItem) { /* happens only when called after start  */
+	strcpy(select, (char *)item);
+	majuscules(select);
+	}
 if(strlen(select) == 0) return;
 trouve = FALSE;
 for(num = 0; num <= totnoms; num++) {
@@ -1539,16 +1642,23 @@ else if(font_num == times)
 int direct_ps_plot(void)
 {
 char *p;
+
 choix = show_tree;
 /* read tree file */
-if( preptree(fname) != NULL ) {
+if( (p = preptree(fname) ) != NULL ) {
+	fprintf(stderr, "%s\n", p);
 	return 1;
 	}
 strcpy(plotfilename, fname);
 p = strchr(plotfilename, '.'); 
 if(p == NULL) p = plotfilename + strlen(plotfilename);
+#ifdef TTY
+strcpy(p, ".txt");
+#else
 strcpy(p, ".ps");
-#ifdef NO_GUI
+#endif
+#ifdef TTY
+#elif defined(NO_GUI)
 	prepare_ps_font(courier, FALSE, FALSE);
 #else
 	prepare_ps_font(times, FALSE, FALSE);
@@ -1557,11 +1667,53 @@ maxy = 1000.;
 deltay = maxy/notu;
 ps_plot = TRUE;
 runtree();
+#ifdef ADDROOT
+/* calcul du nombre de feuilles (tres obligatoire !!!) */
+calc_otu(NULL, racine);
+/*  calcul de son format phylip */
+p = ecrit_arbre_parenth(racine);
+if(p == NULL) {
+	fprintf(stderr, "Sorry, not enough memory\n");
+	exit(1);
+	}
+/* ecriture de son format phylip */
+printf("%s\n", p);
+#elif defined(TTY)
+tty_plot();
+#else
 postscript_plot();
+#endif
 return 0;
 }
 
 
+int calc_otu(struct noeud *pere, struct noeud *centre)
+{
+struct noeud *gauche, *droite;
+double bg, bd, bpere;
+
+if(centre == NULL) return 0;
+
+/* orienter le noeud centre de maniere standard: centre->v3=pere */
+if( centre->v1 == pere ) {
+	gauche =centre->v2; droite = centre->v3;
+	bg = centre->l2; bd = centre->l3; bpere = centre->l1;
+	}
+else if( centre->v2 == pere ) {
+	gauche =centre->v1; droite = centre->v3;
+	bg = centre->l1; bd = centre->l3; bpere = centre->l2;
+	}
+else	{
+	gauche =centre->v1; droite = centre->v2;
+	bg = centre->l1; bd = centre->l2; bpere = centre->l3;
+	}
+centre->v3=pere; centre->v1=gauche; centre->v2=droite;
+centre->l3=bpere; centre->l1=bg; centre->l2=bd;
+
+return calc_otu(centre, centre->v1) + calc_otu(centre, centre->v2);
+}
+
+
 void remove_arg(int target, int *argc, char *argv[])
 {
 int num;
@@ -1571,7 +1723,7 @@ for(num = target; num < *argc - 1; num++)
 }
 
 
-void process_args(int *argc, char *argv[])
+void process_args(int *argc, char *argv[], char **pred)
 {
 int num, taille;
 
@@ -1582,9 +1734,15 @@ for(num = 1; num < *argc; num++) {
 		"Usage: %s [-options] [tree_file_name]\n"
 "where options are:\n"
 "-h             print out this message\n"
+#ifdef ADDROOT
+"-outotu name   name of species to use as outgroup for rooting\n"
+#elif defined(TTY)
+"-txtw n        width of output text (default = 120)\n"
+#else
 "-us            postcript tree file prepared for US Letter paper size\n"
 #ifndef NO_GUI
 "-psonly        no window interface, just write the postscript tree file\n"
+"-red xxxx      highlight in red all taxon names containing string xxxx\n"
 #endif
 "-pc n          number of pages for PostScript output\n"
 "-size n        font size n used for taxon names\n"
@@ -1592,8 +1750,9 @@ for(num = 1; num < *argc; num++) {
 "-boot          show bootstrap values if they appear in tree file\n"
 "-pssize        size of page for postscript expressed as WIDTHxHEIGHT\n"
 "-notitle       don't include title in postscript output\n"
+#endif
 "\n"
-"and where tree_file_name is the name of a tree file using nested parentheses\n" 
+"and where tree_file_name is the name of a tree file in the Newick parentheses format\n" 
 , extract_filename(argv[0]) );
 
 #ifdef MICRO
@@ -1679,6 +1838,42 @@ for(num = 1; num < *argc; num++) {
 		break;
 		}
 	}
+*pred = NULL;
+#ifndef NO_GUI
+for(num = 1; num < *argc; num++) {
+	if( strncmp(argv[num], "-red", 4) == 0) {
+		if(num + 1 < *argc) {
+			*pred = argv[num+1];
+			remove_arg(num + 1, argc, argv);
+			}
+		remove_arg(num, argc, argv);
+		break;
+		}
+	}
+#elif defined(TTY)
+physx = 120; /* default width of text output */
+for(num = 1; num < *argc; num++) {
+	if( strncmp(argv[num], "-txtw", 5) == 0) {
+		if(num + 1 < *argc) {
+			sscanf(argv[num+1], "%d", &physx);
+			remove_arg(num + 1, argc, argv);
+			}
+		remove_arg(num, argc, argv);
+		break;
+		}
+	}
+#elif defined(ADDROOT)
+for(num = 1; num < *argc; num++) {
+	if(strcmp(argv[num], "-outotu") == 0) {
+		if(num + 1 < *argc) {
+			outotu = argv[num+1];
+			remove_arg(num + 1, argc, argv);
+			}
+		remove_arg(num, argc, argv);
+		break;
+		}
+	}
+#endif
 }
 
 
@@ -1762,32 +1957,79 @@ xf = physx * 0.95;
 xd = xf - phys_w;
 sprintf(ech_name, "%.1g", log_val);
 lc = calc_text_size(ech_name, NULL, NULL);
-dir_moveto( (xd + xf)/2 - lc/2, 
-	y + char_height/6. ); 
-plotstring(ech_name);
 dir_moveto(xd, y);
 dir_lineto(xf, y);
+#ifndef TTY
 dir_moveto(xd, y - char_height/3.);
 dir_lineto(xd, y + char_height/3.);
 dir_moveto(xf, y - char_height/3.);
 dir_lineto(xf, y + char_height/3.);
+#endif
+dir_moveto( (xd + xf)/2 - lc/2, 
+	y + char_height/6. ); 
+plotstring(ech_name);
 }
 
 
+#ifndef NO_GUI
+void scrollcallback(Nlm_BaR b, Nlm_SlatE g, Nlm_Int2 after, Nlm_Int2 before)
+{
+tree_draw_proc(fd_nj_plot->tree_plot);
+}
+
+
+void zoomcallback(Nlm_SwitcH s, Nlm_Int2 after, Nlm_Int2 before)
+{
+char aux[50];
+int max, hide;
+
+if(after == 1) fd_nj_plot->zoomvalue = 1.;
+else if(after > before) fd_nj_plot->zoomvalue *= 1.4;
+else if(after < before) fd_nj_plot->zoomvalue /= 1.4;
+sprintf(aux, "%d%%", (int)(100. * fd_nj_plot->zoomvalue + 0.5) );
+Nlm_SetTitle(fd_nj_plot->zoomprompt, aux);
+Nlm_InvalObject(fd_nj_plot->zoomprompt);
+scrollnotu(subtree_notu);
+hide = (after == 1);
+Nlm_SetPanelExtra(fd_nj_plot->tree_plot, &hide);
+tree_draw_proc(fd_nj_plot->tree_plot);
+}
+
+
+void scrollnotu(int count)
+{
+Nlm_BaR obj; int lines;
+float old;
+
+	obj = Nlm_GetSlateVScrollBar( (SlatE)fd_nj_plot->tree_plot );
+	old = Nlm_GetBarValue(obj) / (float)Nlm_GetBarMax(obj);
+	lines = (count + 1) * (1 - 1/fd_nj_plot->zoomvalue) + 0.5; 
+	if(lines < 3) lines = 3;
+	Nlm_CorrectBarMax(obj, lines);
+	Nlm_CorrectBarValue(obj, old * lines);
+	Nlm_CorrectBarPage(obj, lines/fd_nj_plot->zoomvalue, lines/fd_nj_plot->zoomvalue);
+}
+
+#endif
+
 void do_plot(void)
 {
 struct trait *p;
 int num, erreur = FALSE;
 int offset;
-double factor, value;
+double factor, value, m = 1, v = 0, zoom = 1;
 
 #ifndef NO_GUI
 /* size of the tree plot window  */
 if( ! (ps_plot || doing_print) ) {
-		Nlm_RecT ob_rect;
-		ObjectRect(fd_nj_plot->tree_plot, &ob_rect);
+		Nlm_RecT ob_rect; Nlm_BaR bar;
+		Nlm_ObjectRect(fd_nj_plot->tree_plot, &ob_rect);
+		bar = Nlm_GetSlateVScrollBar((Nlm_SlatE)fd_nj_plot->tree_plot);
+		v = (double)Nlm_GetBarValue(bar);
+		m = (double)Nlm_GetBarMax(bar);
 		physx = ob_rect.right - ob_rect.left;
-		physy = ob_rect.bottom - ob_rect.top;
+		physy = (ob_rect.bottom - ob_rect.top);
+		zoom = fd_nj_plot->zoomvalue;
 	}
 #endif
 
@@ -1806,7 +2048,15 @@ if(physx_corr <= physx_min + 1) {
 	}
 physy_corr = physy - 2*char_height;
 physy_min  = char_height;
-scale_window(0, maxx, 0, maxy, 
+#ifdef TTY
+physy_corr = physy - 3;
+physy_min  = 0;
+#endif
+/* scale_window(0, maxx, 0, maxy, 
+	physx_min, physx_corr, physy_min, physy_corr); */
+scale_window(0, maxx, 
+	((m-v)/m) * (maxy - maxy/zoom), 
+	((m-v)/m) * (maxy - maxy/zoom) + maxy/zoom, 
 	physx_min, physx_corr, physy_min, physy_corr);
 
 for(num=0; num<=totnoms; num++) {
@@ -1844,6 +2094,9 @@ if(ps_plot) {
 	ascent = macfontinfo.ascent;
 	descent = macfontinfo.descent;
 	}
+#elif defined(TTY)
+	width = strlen(text);
+	ascent = 1; descent = 0;
 #elif defined(NO_GUI)
 	width = strlen(text) * font_size * postscript_ratio + 0.5;
 	ascent = font_size; descent = 0;
@@ -1890,6 +2143,8 @@ if(ps_plot) {
 	*copy = l;
 	memcpy(copy + 1, nom, l + 1);
 	DrawString( (ConstStr255Param) copy);
+#elif defined(TTY)
+	memcpy(tty_page[tty_y]+tty_x+1, nom, strlen(nom));
 #else
 	fprintf(plotfile,"(%s) show\n",nom);
 #endif
@@ -1918,6 +2173,8 @@ if(ps_plot) {
 	xi += margin;
 	yi = physy - y + margin;
 	MoveTo(xi, yi);
+#elif defined(TTY)
+	tty_x = x + 0.5; tty_y = y + 0.5;
 #else
 	yi=y;
 	fprintf(plotfile,"%d %d moveto\n",xi,yi);
@@ -1946,12 +2203,34 @@ void dir_lineto(double x,double y)
 /* draw line from current pos to physical coord x,y */
 {
 int xi,yi;
+char *p;
 if(ps_plot) {
 	xi=x;
 #ifdef WIN_MAC
 	xi += margin;
 	yi = physy - y + margin;
 	LineTo(xi, yi);
+#elif defined(TTY)
+	if(tty_y == (int)(y + 0.5)) {
+		xi = x + 0.5;
+		while(tty_x != xi) { 
+			p = tty_page[tty_y] + tty_x; 
+			*p = (*p == '|' ? '+' : '-'); 
+			if(tty_x < xi) ++tty_x; else --tty_x;
+			}
+		p = tty_page[tty_y] + tty_x; 
+		*p = (*p == '|' ? '+' : '-'); 
+		}
+	else {
+		yi = y + 0.5;
+		while(tty_y != yi) { 
+			p = tty_page[tty_y] + tty_x; 
+			*p = (*p == '-' ? '+' : '|'); 
+			if(tty_y < yi) ++tty_y; else --tty_y;
+			}
+		p = tty_page[tty_y] + tty_x; 
+		*p = (*p == '-' ? '+' : '|'); 
+		}
 #else
 	yi=y;
 	fprintf(plotfile,"%d %d lineto stroke\n",xi,yi);
@@ -1976,6 +2255,16 @@ else	{
 
 
 #ifdef WIN_MAC
+
+int is_macosx(void)
+{
+long val;
+
+Gestalt (gestaltSystemVersion, &val);
+return (val >= 0x1000);
+}
+
+
 int crefpict(char *fname, PicHandle picture)
 {
 OSErr success = 0, err;
@@ -2244,9 +2533,9 @@ ArrowCursor();
 
 char *preptree(char *fname)
 {
-int i, c, steparbre, maxarbre, maxlname;
+int i, c, steparbre, maxarbre, maxlname, v;
 FILE *njfile;
-char *arbre, *der_arbre, *finarbre, *tmp;
+char *arbre, *der_arbre, *finarbre, *tmp, *p;
 static char message[200];
 
 if( (njfile=fopen(fname,"r")) == NULL ){
@@ -2255,11 +2544,11 @@ if( (njfile=fopen(fname,"r")) == NULL ){
 	}
 /* recherche du debut de la description de l'arbre */
 do c=fgetc(njfile);
-while (c == '\n' || c == ' ' );
+while ( isspace(c) );
 /* for fastDNAml format, skip initial [comment] */
 if(c == '[') {
 	do c=fgetc(njfile); while (c != ']');
-	do c=fgetc(njfile); while (c == '\n' || c == '\r' || c == ' ' );
+	do c=fgetc(njfile); while ( isspace(c) );
 	}
 if ( c != '(') {
 	fclose(njfile);
@@ -2271,7 +2560,7 @@ if ( c != '(') {
 	arbre=check_alloc(maxarbre,1);
 	der_arbre = arbre+maxarbre;
 	*arbre=c;
-	notu=2; i=3;
+	notu=2; i=3; v = 0;
 	finarbre=arbre;
 	while( (c=fgetc(njfile)) != EOF && c != ';') {
 		if( c=='\n' || c=='\r') continue;
@@ -2285,16 +2574,25 @@ if ( c != '(') {
 			der_arbre=arbre+maxarbre;
 			}
 		*(finarbre)=c;
-		if(c==')')notu++;
-		if(c=='(')i++;
+		if(c == ')') notu++;
+		if(c == '(') i++;
+		if(c == ',') v++;
 		}
 	*(finarbre+1)='\0';
-long_arbre_parenth= (finarbre-arbre+1);
 fclose(njfile);
 if(i!=notu)goto erreur;
-notu--;
-totbranches= -1;
 
+arbre = (char *)realloc(arbre, strlen(arbre) + 4 * v + 5 ); /* worst case add 4 chars for each , */
+make_binary_or_unrooted(arbre);
+long_arbre_parenth = strlen(arbre);
+	notu = 2; p = arbre;
+	while( *p != 0 ) {
+		if( *p == ')')notu++;
+		p++;
+		}
+
+notu--; /* after this notu = number of OTUs - 1 for unrooted trees */
+totbranches= -1;
 /* allocate all memory */
 tabtax = (struct noeud **)check_alloc(2*notu +1,sizeof(struct noeud *));
 if(notu > 2) branches = (branche *)check_alloc(notu-2,sizeof(branche));
@@ -2347,6 +2645,10 @@ else	{
 		}
 	}
 if(notu+1<3) return ("Tree should contain at least 3 elements.");
+subtree_notu = notu;
+#ifndef NO_GUI
+scrollnotu(notu);
+#endif
 return NULL;
 
 erreur:
@@ -2404,6 +2706,7 @@ else	{
 /*!!!!!!!! notu computed before was 1 unit too high !!!!!!!!*/
 	notu--;
 	}
+
 num_noeud=notu;
 nextotu= -1;
 p1=unrootedset(deba,debb-2,&int_br_g);
@@ -2562,11 +2865,88 @@ exit(1);
 }
 
 
+
+void make_binary_or_unrooted(char *arbre)
+{
+char *finarbre, *deba, *debb, *debc;
+
+finarbre= nextpar(arbre);
+*(finarbre + 1) = 0;
+deba=arbre+1;
+debb=deba;
+while(*debb != ',') {
+	if(*debb == 0) bad_format("Incorrect tree file");
+	if(*debb == '(')debb=nextpar(debb);
+	debb++;
+	}
+debb++;
+debc=debb;
+while(*debc != ',' && debc<finarbre) {
+	if(*debc == '(')debc=nextpar(debc);
+	debc++;
+	}
+if(*debc == ',') {
+/* the tree is unrooted <==> it has 3 subtrees or more at its bottommost level */
+	make_binary(arbre , debb, finarbre - 1, TRUE);
+	make_binary(arbre , deba, debb - 2, TRUE);
+	}
+else make_binary(arbre , deba, finarbre - 1, TRUE);
+return;
+}
+
+
+int make_binary(char *arbre, char *debut, char *fin, int go_down)
+{
+int virg, l, retval;
+char *p, *q;
+
+p = debut; virg = 0; retval = 0;
+while(p < fin) {
+	if(*p == ',') virg++;
+	else if(*p == '(')	{
+		q = nextpar(p);
+		if(go_down) {
+			l = make_binary(arbre, p+1, q-1, TRUE);
+			fin += l;
+			retval += l;
+			p = q + l;
+			}
+		else p = q;
+		}
+	p++;
+	}
+if(virg > 1) {
+	/* recherche de la 2eme virgule */
+	p = debut; l = 0;
+	while(TRUE) {
+		if(*p == ',') {
+			l++;
+			if(l == 2) break;
+			}
+		else if(*p == '(')	{
+			p = nextpar(p);
+			}
+		p++;
+		}
+	l = strlen(p);
+	memmove(p + 4, p, l + 1);
+	memmove(debut + 1, debut, p - debut);
+	*debut = '(';
+	memcpy(p + 1, "):0", 3);
+	fin += 4;
+	retval += 4;
+	if(virg > 2) retval += make_binary(arbre, debut, fin, FALSE);  
+	}
+return retval;
+}
+
+
 void mydrawstring(double x, double y, char *nom, char option)
 {
 static double px,py;
 
 ch_echelle(x,y,&px,&py);
+#ifndef TTY
 if(option == '1') {
 /* ecrire une chaine en la montant d'un chouia */
 	py += char_height/6.;
@@ -2582,6 +2962,7 @@ else if(option == 't')	{
 else if(option == 'b')	{
 	py += char_height/6.;
 	}
+#endif
 dir_moveto(px,py);
 #ifndef NO_GUI
 if(option == 'r') 
@@ -2668,8 +3049,21 @@ struct noeud *aux;
 double laux;
 
 current_best_diff = VERY_BIG;
-current_cote1=current_cote2=NULL;
-parcourir_branches(*tabtax,NULL);
+if(outotu != NULL) {
+	int i; 
+	for(i=0; i <= notu; i++) {
+		if(labels[i] != NULL && strcmp(labels[i], outotu) == 0) break;	
+		}
+	if(i > notu) {
+		fprintf(stderr, "Error: outotu %s not found in tree\n", outotu);
+		exit(1);
+		}
+	process_branche(tabtax[i], tabtax[i]->v3, tabtax[i]->l3);
+	}
+else {
+	current_cote1=current_cote2=NULL;
+	parcourir_branches(*tabtax, NULL);
+	}
 rooted = TRUE;
 root_br_l = current_br_length;
 /* il faut toujours que la racine soit telle que racine->v1->v3=racine */
@@ -2939,7 +3333,7 @@ for(i=0; i<=notu; i++) {
 	j = strlen(tabnames[i]);
 	strcpy(tabnames[i] + j, labels[i]);
 	}
-
+#ifndef ADDROOT
 totnoms = tottraits = totpoints = -1;
 end_br_length = br_length_txt;
 currx = 0.;
@@ -2947,8 +3341,10 @@ maxx = 0.;
 memset(profs, 0, (notu+1)*sizeof(double));
 if(subtree_center != NULL)
 	deltay = maxy / subtree_notu;
-else
+else {
 	deltay = maxy / notu;
+	subtree_notu = notu;
+	}
 nexty = -deltay;
 calc_text_size("Mq", &char_height, &ascent);
 if(subtree_center == NULL)
@@ -2956,6 +3352,7 @@ if(subtree_center == NULL)
 else
 	mem_plot(subtree_ascend, subtree_center, currx, &curry);
 converted = FALSE;
+#endif
 } /* end of runtree */
 
 
@@ -3384,6 +3781,52 @@ if(!ps_plot_only ) {
 }
 
 
+
+void tty_plot(void)
+{
+double currx, curry;
+int h, erreur = FALSE;
+char *p;
+
+if(notu == 0) return;
+ps_plot = TRUE;
+
+/* physx was set in process_args */
+physy = 2*(notu + 1) + 4;
+tty_page = (char **)malloc(physy*sizeof(char *));
+for(h=0; h < physy; h++) {
+	tty_page[h] = (char *)malloc(physx + 1);
+	memset(tty_page[h], ' ', physx);
+	tty_page[h][(int)physx] = 0;
+	}
+
+totnoms = tottraits = totpoints = -1;
+end_br_length = br_length_txt;
+currx = 0.;
+maxx = 0.;
+nexty = -deltay;
+if(subtree_center == NULL)
+	mem_plot(NULL, racine, currx, &curry);
+else
+	mem_plot(subtree_ascend, subtree_center, currx, &curry);
+do_plot();
+for(h=0; h < physy; h++) {
+	p = tty_page[h] + strlen(tty_page[h]) - 1;
+	while(p > tty_page[h] && *p == ' ') *(p--) = 0;
+	}
+
+plotfile=fopen(plotfilename, "w");
+for(h=1; h <= physy; h++) {
+	fprintf(plotfile, "%s\n", tty_page[(int)physy - h]);
+	}
+if(ferror(plotfile)) erreur = TRUE;
+if(fclose(plotfile) != 0) erreur = TRUE;
+
+need_runtree = TRUE;
+ps_plot = FALSE;
+}
+
+
 void majuscules(char *p)
 {
 while(*p != 0) {*p = toupper(*p); p++;}
@@ -3406,3 +3849,43 @@ return name;
 }
 
 
+#ifdef WIN_MOTIF
+#include <ncbiport.h>
+
+
+static void Nlm_GetFontData (Nlm_FonT f, Nlm_FontData * fdata)
+
+{
+  Nlm_FntPtr  fp;
+
+  if (f != NULL && fdata != NULL) {
+    fp = (Nlm_FntPtr) Nlm_HandLock (f);
+    *fdata = *fp;
+    Nlm_HandUnlock (f);
+  }
+}
+
+
+void set_systemfont(char *fname)
+{
+		XFontStruct   *font;
+		XmFontListEntry entry;
+		Nlm_FontData fontdata;
+		extern XmFontList Nlm_XfontList;
+		
+		Nlm_systemFont = Nlm_ParseFont(fname);
+        Nlm_GetFontData(Nlm_systemFont, &fontdata);
+        font= (XFontStruct *)fontdata.handle;
+		entry = XmFontListEntryCreate(XmFONTLIST_DEFAULT_TAG, 
+					XmFONT_IS_FONT, (XtPointer)font);
+        Nlm_XfontList = XmFontListAppendEntry(NULL, entry);
+                
+        Nlm_SelectFont(Nlm_systemFont);
+        Nlm_stdLineHeight=Nlm_LineHeight();
+        Nlm_stdFontHeight=Nlm_FontHeight();
+        Nlm_stdAscent=Nlm_Ascent();
+        Nlm_stdDescent=Nlm_Descent();
+        Nlm_stdLeading=Nlm_Leading();
+        Nlm_stdCharWidth = Nlm_MaxCharWidth ();
+}
+#endif
