@@ -7,13 +7,23 @@
 #define TRUE (!FALSE)
 #endif
 
-#ifdef __MWERKS__
-#define M_PI _PI
+#ifndef M_PI
+#define M_PI           3.14159265358979323846  /* pi */
+#endif
+#ifndef M_PI_2
 #define	M_PI_2		(M_PI/2)
 #endif
 
 
 /* typedefs */
+
+typedef struct {
+	unsigned int left : 1;
+	unsigned int right : 1;
+	unsigned int top : 1;
+	unsigned int bottom : 1;
+	} hits_field;
+
 struct noeud {
 	double l1,l2,l3;
 	struct noeud *v1,*v2,*v3;
@@ -32,6 +42,7 @@ typedef struct _cp_point {
 
 typedef struct _branche {
 	cp_point debut, fin;
+        int color;
 	char *nom;
 	} branche;
 
@@ -84,8 +95,7 @@ branche *calc_position_noeuds(bignoeud *debut, bignoeud *parent,
 void mem_line(cp_point *debut, cp_point *fin, bignoeud *noeud_term, 
 	branche *br);
 void draw_tree(plot_data *ob, branche *branches, int comp_phys_bounds);
-void square_phys(plot_data *ob, char xy);
-int chg_phys(plot_data *ob, hit kind);
+int chg_phys(plot_data *ob, hits_field hits, int char_height);
 void log_to_phys(cp_point *log_pos, cp_point *phys_pos);
 double length_log_phys(double p);
 double length_phys_log(double p);
@@ -94,10 +104,10 @@ double length_phys_log(double p);
 char *check_alloc(int nbrelt, int sizelt);
 void err_message(char *text);
 hit draw_line(branche *br, plot_data *ob, int doit, int char_width,
-	int char_height, int descend);
+	int char_height, int descend, int ascent);
 int my_get_char_height(int *ascend, int *descend);
 int my_get_string_width(char *nom);
-void draw_scale(plot_data *ob);
+void draw_scale(plot_data *ob, int char_height, int ascent);
 
 /* globals */
 struct noeud *racine;
@@ -164,13 +174,16 @@ totbranches= -1;
 
 /* allocate all memory */
 tabtax = (struct noeud **)check_alloc(2*notu +1,sizeof(struct noeud *));
-branches = (branche_noeud *)check_alloc(notu-2,sizeof(branche_noeud));
+if(notu > 2) branches = 
+	(branche_noeud *)check_alloc(notu-2, sizeof(branche_noeud));
 for(i=0; i<2*notu+1; i++) *(tabtax+i)=
 			(struct noeud *)check_alloc(1,s_noeud);
 loadphylip(arbre);
 free(arbre);
+if(notu == 0) goto erreur;
 if(!rooted) {
 	racine = *(tabtax+(++num_noeud));
+	if(num_noeud >= 2*notu + 1) return("Error: incorrect tree file");
 	if(has_br_length) {
 		place_midpoint_root();
 		}
@@ -240,7 +253,11 @@ else	{
 num_noeud=notu;
 nextotu= -1;
 p1=unrootedset(arbre,deba,debb-2,&int_br_g);
-p2=unrootedset(arbre,debb,debc-2,&int_br_d);
+if(p1 != NULL) p2=unrootedset(arbre,debb,debc-2,&int_br_d);
+if(p1 == NULL || p2 == NULL || num_noeud + 1 >= 2*notu + 1) {
+	notu = 0;
+	return;
+	}
 p = *(tabtax+(++num_noeud));
 if(!has_br_length) {
 	p1->l3 = 0.5*p1->l3;
@@ -252,6 +269,10 @@ p->v2=p2; p2->v3=p; p->l2=p2->l3;
 if(int_br_d!=NULL) { int_br_d->bouta=p; int_br_d->boutb=p2; }
 if(!rooted) {
 	p3=unrootedset(arbre,debc,finarbre-1,&int_br_g);
+	if(p3 == NULL ) {
+		notu = 0;
+		return;
+		}
 	if(int_br_g!=NULL) { int_br_g->bouta=p; int_br_g->boutb=p3; }
 	p->v3=p3; p3->v3=p; p->l3=p3->l3;
 	}
@@ -292,7 +313,10 @@ if(virg>fin) virg=deb;
 if(*virg == ',') {
 	struct noeud *p1,*p2;
 	p1=unrootedset(arbre,deb,virg-1,&int_br_g);
+if(p1 == NULL) return NULL;
 	p2=unrootedset(arbre,virg+1,fin,&int_br_d);
+if(p2 == NULL) return NULL;
+if(num_noeud + 1 >= 2*notu + 1) return NULL;
 	p = *(tabtax+(++num_noeud));
 	p->v1=p1; p1->v3=p; p->l1=p1->l3;
 	if(int_br_g!=NULL) { int_br_g->bouta=p; int_br_g->boutb=p1; }
@@ -323,6 +347,7 @@ else	{
 		branche_noeud *prov;
 		fpar=nextpar(arbre,deb)-1;
 		p=unrootedset(arbre,deb+1,fpar,&prov);
+if(p == NULL) return NULL;
 /* recherche internal label */
 		l=virg-fpar-1;
 		if(l>0) {
@@ -341,7 +366,7 @@ else	{
 		++nextotu;
 		p= *(tabtax+nextotu);
 		p->nom = (char *)check_alloc(n+1,1);
-		memcpy(p->nom, deb, n); p->nom[n+1] = 0;
+		memcpy(p->nom, deb, n); p->nom[n] = 0;
 		p->v1=p->v2=p->v3=NULL;
 		}
 	p->l3=brlength;
@@ -546,8 +571,12 @@ else	{
 		gauche =racine->v1; droite = racine->v2;
 		bg = (racine->l1); bd = (racine->l2);
 		}
-	lg = get_length_down(racine,gauche) + bg;
-	ld = get_length_down(racine,droite) + bd;
+ 	/* conserver cette ecriture sinon plante sur PC */
+ 	lg = ld = 0;
+ 	if(gauche != NULL) lg = get_length_down(racine,gauche);
+ 	lg += bg;
+ 	if(droite != NULL) ld = get_length_down(racine,droite);
+ 	ld += bd;
 	return ((lg+ld)/2);
 	}
 }
@@ -624,17 +653,20 @@ double calc_dist_centre_feuilles(bignoeud *debut, bignoeud *parent)
 {
 double valeur, current;
 if(debut == NULL) return 0;
-valeur=0;
+valeur = 0;  /*  !!!!! conserver cette ecriture sinon plante sur PC !!!!!! */
 if(debut->v1 != parent) {
-	current = debut->l1 + calc_dist_centre_feuilles(debut->v1, debut);
+	current = calc_dist_centre_feuilles(debut->v1, debut);
+	current += debut->l1;
 	if(current > valeur) valeur = current;
 	}
 if(debut->v2 != parent) {
-	current = debut->l2 + calc_dist_centre_feuilles(debut->v2, debut);
+	current = calc_dist_centre_feuilles(debut->v2, debut);
+	current += debut->l2;
 	if(current > valeur) valeur = current;
 	}
 if(debut->v3 != parent) {
-	current = debut->l3 + calc_dist_centre_feuilles(debut->v3, debut);
+	current = calc_dist_centre_feuilles(debut->v3, debut);
+	current += debut->l3;
 	if(current > valeur) valeur = current;
 	}
 return valeur;
@@ -818,77 +850,163 @@ br->nom = noeud_term->nom;
 
 void draw_tree(plot_data *ob, branche *branches, int comp_phys_bounds)
 {
-int char_width, char_height, num, dernier, limite = FALSE, ascend, descend;
+int char_width, char_height, num, dernier, encore = FALSE, ascend, descend;
 hit result;
+hits_field hits;
 char_width = my_get_string_width("M");
 char_height = my_get_char_height(&ascend, &descend);
 dernier = 2*(notu+1)-3;
 if( comp_phys_bounds ) {
+/* rendre dimensions carrees */
+	if (phys_max_x - phys_min_x > phys_max_y - phys_min_y)
+		phys_max_x = phys_min_x + (phys_max_y - phys_min_y);
+	else if(phys_max_y - phys_min_y > phys_max_x - phys_min_x)
+		phys_max_y = phys_min_y + (phys_max_x - phys_min_x);
+
 	do	{
-		for(num=0; num< dernier; num++) {
-			result = draw_line(branches+num, ob, FALSE, char_width, char_height, descend);
-			if(result != NO_HIT) break;
+		hits.left = hits.right = hits.top = hits.bottom = 0;
+		for(num = 0; num < dernier; num++) {
+			result = draw_line(branches+num, ob, FALSE, 
+				char_width, char_height, descend, ascend);
+			if(result == LEFT_HIT) hits.left = 1;
+			else if(result == RIGHT_HIT) hits.right = 1;
+			else if(result == TOP_HIT) hits.top = 1;
+			else if(result == BOTTOM_HIT) hits.bottom = 1;
 			}
-		if(result != NO_HIT) limite = chg_phys(ob, result);
+		encore = chg_phys(ob, hits, char_height);
 		}
-	while (result != NO_HIT && !limite);
+	while (encore);
+	for(num = 0; num < dernier; num++) branches[num].color = FALSE;
 	}
 for(num=0; num< dernier; num++) {
-	draw_line(branches+num, ob, TRUE, char_width, char_height, descend);
+	draw_line(branches+num, ob, TRUE, char_width, char_height, 
+		descend, ascend);
 	}
-if(has_br_length) draw_scale(ob);
+if(has_br_length) draw_scale(ob, char_height, ascend);
 }
 
 
-void square_phys(plot_data *ob, char xy)
-{
-int h, w;
-w = phys_max_x - phys_min_x +1;
-h = phys_max_y - phys_min_y +1;
-if( xy == 'x' && w > h ) {
-	w = h;
-	}
-else if( xy == 'y' && h > w ) {
-	h = w;
-	}
-else
-	return;
-phys_min_x = (ob->w)/2 - w/2;
-phys_max_x = phys_min_x + w - 1;
-phys_min_y = (ob->h)/2 - h/2;
-phys_max_y = phys_min_y + h - 1;
-}
 
-
-int chg_phys(plot_data *ob, hit kind)
+int chg_phys(plot_data *ob, hits_field hits, int char_height)
 {
-int newval;
+int encore = FALSE;
 const float delta = 0.05;
-if(kind == TOP_HIT) {
-	newval = phys_min_y + (ob->h * delta);
-	if(newval >= phys_max_y) return TRUE;
-	phys_min_y = newval;
-	square_phys(ob, 'x');
+int value;
+static int lr = 0, tb = 0, oldvaluex = 0, oldvaluey = 0;
+
+if(hits.left == 1 && hits.right == 0) {
+	if(oldvaluex == 0) value = (phys_max_x - phys_min_x) * delta; 
+	else value = oldvaluex;
+	if(value < 2) value = 2;
+	if(lr == 2) { /* alternance left/right */
+		if(oldvaluex <= 2) {
+			hits.right = 1;
+			return chg_phys(ob, hits, char_height);
+			}
+		value = oldvaluex/2; 
+		}
+	lr = 1;
+	oldvaluex = value;
+	phys_min_x += value;
+	phys_max_x += value;
+	if(phys_max_x <= phys_min_x) {
+		phys_max_x = phys_min_x + 1;
+		return FALSE;
+		}
+	encore = TRUE;
 	}
-else if(kind == BOTTOM_HIT) {
-	newval = phys_max_y - (ob->h * delta);
-	if(newval <= phys_min_y) return TRUE;
-	phys_max_y = newval;
-	square_phys(ob, 'x');
+else if(hits.left == 0 && hits.right == 1) {
+	if(oldvaluex == 0) value = (phys_max_x - phys_min_x) * delta; 
+	else value = oldvaluex;
+	if(value < 2) value = 2;
+	if(lr == 1) { /* alternance left/right */
+		if(oldvaluex <= 2) {
+			hits.left = 1;
+			return chg_phys(ob, hits, char_height);
+			}
+		value = oldvaluex/2; 
+		}
+	lr = 2;
+	oldvaluex = value;
+	phys_max_x -= value;
+	phys_min_x -= value;
+	if(phys_max_x <= phys_min_x) {
+		phys_max_x = phys_min_x + 1;
+		return FALSE;
+		}
+	encore = TRUE;
 	}
-else if(kind == LEFT_HIT) {
-	newval = phys_min_x + (ob->w * delta);
-	if(newval >= phys_max_x) return TRUE;
-	phys_min_x = newval;
-	square_phys(ob, 'y');
+else if(hits.left == 1 && hits.right == 1) {
+	lr = tb = 0;
+	oldvaluex = 0;
+	value = (phys_max_x - phys_min_x) * delta / 2; if(value < 2) value = 2;
+	phys_min_x += value;
+	phys_max_x -= value;
+	if(phys_max_x <= phys_min_x) {
+		phys_max_x = phys_min_x + 1;
+		return FALSE;
+		}
+	encore = TRUE;
+	phys_max_y = phys_min_y + (phys_max_x - phys_min_x);
 	}
-else if(kind == RIGHT_HIT) {
-	newval = phys_max_x - (ob->w * delta);
-	if(newval <= phys_min_x) return TRUE;
-	phys_max_x = newval;
-	square_phys(ob, 'y');
+
+if(hits.top == 0 && hits.bottom == 1) {
+	if(oldvaluey == 0) value = (phys_max_y - phys_min_y) * delta; 
+	else value = oldvaluey;
+	if(value < 2) value = 2;
+	if(tb == 2) {
+		if(oldvaluey <= 2) {
+			hits.top = 1;
+			return chg_phys(ob, hits, char_height);
+			}
+		value = oldvaluey/2; 
+		}
+	tb = 1;
+	oldvaluey = value;
+	phys_max_y -= value;
+	phys_min_y -= value;
+	if(phys_max_y <= phys_min_y) {
+		phys_max_y = phys_min_y + 1;
+		return FALSE;
+		}
+	encore = TRUE;
 	}
-return FALSE;
+else if(hits.top == 1 && hits.bottom == 0) {
+	if(oldvaluey == 0) value = (phys_max_y - phys_min_y) * delta; 
+	else value = oldvaluey;
+	if(value < 2) value = 2;
+	if(tb == 1) {
+		if(oldvaluey <= 2) {
+			hits.bottom = 1;
+			return chg_phys(ob, hits, char_height);
+			}
+		value = oldvaluey/2; 
+		}
+	tb = 2;
+	oldvaluey = value;
+	phys_min_y += value;
+	phys_max_y += value;
+	if(phys_max_y <= phys_min_y) {
+		phys_max_y = phys_min_y + 1;
+		return FALSE;
+		}
+	encore = TRUE;
+	}
+else if(hits.top == 1 && hits.bottom == 1) {
+	value = (phys_max_y - phys_min_y) * delta/2; if(value < 2) value = 2;
+	lr = tb = 0;
+	oldvaluey = 0;
+	phys_min_y += value;
+	phys_max_y -= value;
+	if(phys_max_y <= phys_min_y) {
+		phys_max_y = phys_min_y + 1;
+		return FALSE;
+		}
+	encore = TRUE;
+	phys_max_x = phys_min_x + (phys_max_y - phys_min_y);
+	}
+
+return encore;
 }
 
 
