@@ -46,6 +46,8 @@ return getche();
 #define DIR_DELIM '/'
 #undef Boolean
 #include <X11/Intrinsic.h>
+void set_systemfont(char *systemfont);
+extern Nlm_CharPtr Nlm_XrmGetResource(const Nlm_Char PNTR _resource);
 
 
 extern XtAppContext Nlm_appContext;
@@ -296,6 +298,12 @@ else    {
         else exit(1);
         }
    Nlm_WatchCursor();
+#ifdef WIN_MOTIF
+	{char *font;
+	font = Nlm_XrmGetResource("systemfont");
+	if(font != NULL) set_systemfont(font);
+	}
+#endif
    fd_unrooted = create_win_unrooted();
    prepare_fonts();
    fd_unrooted->tree_name = fname;
@@ -1535,3 +1543,44 @@ doing_print = FALSE;
 
 #endif
 
+#ifdef WIN_MOTIF
+#include <ncbiport.h>
+
+
+static void Nlm_GetFontData (Nlm_FonT f, Nlm_FontData * fdata)
+
+{
+  Nlm_FntPtr  fp;
+
+  if (f != NULL && fdata != NULL) {
+    fp = (Nlm_FntPtr) Nlm_HandLock (f);
+    *fdata = *fp;
+    Nlm_HandUnlock (f);
+  }
+}
+
+
+void set_systemfont(char *fname)
+{
+		XFontStruct   *font;
+		XmFontListEntry entry;
+		Nlm_FontData fontdata;
+		extern XmFontList Nlm_XfontList;
+		
+		Nlm_systemFont = Nlm_ParseFont(fname);
+        Nlm_GetFontData(Nlm_systemFont, &fontdata);
+        font= (XFontStruct *)fontdata.handle;
+		entry = XmFontListEntryCreate(XmFONTLIST_DEFAULT_TAG, 
+					XmFONT_IS_FONT, (XtPointer)font);
+        Nlm_XfontList = XmFontListAppendEntry(NULL, entry);
+                
+        Nlm_SelectFont(Nlm_systemFont);
+        Nlm_stdLineHeight=Nlm_LineHeight();
+        Nlm_stdFontHeight=Nlm_FontHeight();
+        Nlm_stdAscent=Nlm_Ascent();
+        Nlm_stdDescent=Nlm_Descent();
+        Nlm_stdLeading=Nlm_Leading();
+        Nlm_stdCharWidth = Nlm_MaxCharWidth ();
+}
+#endif
+
